require 'spec_helper'
require 'thread'

Thread.abort_on_exception = true

describe Blender::Lock::Zk do

  context 'raw api' do
    it 'without timeout' do
      t = Thread.new do
        l1 = Blender::Lock::Zk.new( '/a/b/c')
        l1.with_lock do
          sleep 2
        end
      end
      sleep 1

      l2 = Blender::Lock::Zk.new( '/a/b/c')
      expect do
        l2.with_lock do
          puts 'Unreachable code'
        end
      end.to raise_error(Blender::LockAcquisitionError)
      t.join
    end

    it 'with timeout' do
      t = Thread.new do
        l1 = Blender::Lock::Zk.new( '/a/b/c')
        l1.with_lock do
          sleep 2
        end
      end
      sleep 1

      l2 = Blender::Lock::Zk.new( '/a/b/c', timeout: 3)
      expect do
        l2.with_lock do
          x = 1 # Solution to NP hard problems
        end
      end.not_to raise_error
      t.join
    end
  end

  context 'with blender dsl' do

    it 'lock without timeout inside dsl' do
      t = Thread.new do
        Blender.blend('test-1') do |sched|
          sched.members(['localhost'])
          sched.lock_options('zk')
          sched.ruby_task('date') do
            execute do
              sleep 2
            end
          end
        end
      end
      sleep 1
      expect do
        Blender.blend('test-1') do |sched|
          sched.members(['localhost'])
          sched.lock_options('zk')
          sched.ruby_task('date') do
            execute do
              puts 'Should fail'
            end
          end
        end
      end.to raise_error(Blender::LockAcquisitionError)
      t.join
    end

    it 'lock with timeout' do
      t = Thread.new do
        Blender.blend('test-1') do |sched|
          sched.members(['localhost'])
          sched.lock_options('zk')
          sched.ruby_task('date') do
            execute do
              sleep 2
            end
          end
        end
      end
      sleep 1
      expect do
        Blender.blend('test-1') do |sched|
          sched.members(['localhost'])
          sched.lock_options('zk', timeout: 5)
          sched.ruby_task('date') do
            execute do
              x = 2 # Another solution to NP hard problem
            end
          end
        end
      end.not_to raise_error
      t.join
    end
  end
end
